/*
 * grader-io - gcp.h
 *
 * Grader Control Protocol
 *
 * Copyright (C) 2016, 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef GCP_H_INCLUDED
#define GCP_H_INCLUDED

/*
 * The Grader Control Protocol uses 7-byte fixed-size packets
 * with 4-bit protocol number and 4-bit node address.
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  | data...                                                       |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 4  |    padding    | proto |  node |           data                |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

#define GCP_PAYLOAD		0
#define GCP_HEADER		1

#define GCP_NODE_SHIFT		16
#define GCP_NODE_MASK		0x0f
#define GCP_NODE_BROADCAST	0x0f

#define GCP_PROTO_SHIFT		20
#define GCP_PROTO_MASK		0x0f
#define GCP_PROTO_SEQCMD	0

/*
 * SEQCMD protocol - reliable execution of commands
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  | data...                                                       |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 4  |    padding    | proto |  node |A|R|SEQ|        command        |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

#define GCP_SEQCMD_CMD_SHIFT	0
#define GCP_SEQCMD_CMD_MASK	0x0fff

#define GCP_SEQCMD_SEQ_SHIFT	12
#define GCP_SEQCMD_SEQ_MASK	0x3
#define GCP_SEQCMD_RST		0x4000
#define GCP_SEQCMD_ACK		0x8000

#define GCP_CMD_BROADCAST	0x8000

/*
 * GCP_CERROR - report command execution error
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |               error code              |        command        |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
#define GCP_CERROR		0x000

/*
 * GCP_CGET - get 32-bit object value
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |                         object id                             |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
#define GCP_CGET		0x001

#define GCP_GET_STATUS		0
#define GCP_GET_SERIAL_IRQ_TIME	0xffff0000

/*
 * GCP_CSP - enter In-System Programming mode
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |                       GCP_CMD_ISP_MAGIC                       |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
#define GCP_CISP		0x002
#define GCP_CISP_MAGIC		0x0a505349

/*
 * GCP_CECHO - echo request
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |                              data                             |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
#define GCP_CECHO		0x003

/*
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |     output    |                 position                      |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * GCP_CADD_EVENT - add event
 * output - output number
 * position - event position
 *
 * GCP_CPEVENT_INFO - inform about periodic event
 * output - event bitmap
 * position - event position
 */
#define GCP_CADD_EVENT		0x004	/* position */
#define GCP_CPEVENT_INFO	0x005	/* position */
#define GCP_EVENT_VALUE_SHIFT	0
#define GCP_EVENT_VALUE_BITS	24
#define GCP_EVENT_VALUE_MASK	((1 << GCP_EVENT_VALUE_BITS) - 1)
#define GCP_EVENT_OUTPUT_SHIFT	24
#define GCP_EVENT_OUTPUT_BITS	8
#define GCP_EVENT_OUTPUT_MASK	((1 << GCP_EVENT_OUTPUT_BITS) - 1)

/*
 * Grader periodic scheduling interface
 *
 * All functions use following interface
 *
 *     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 0  |W|    id       |                    value                      |
 *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * W - write bit, 0 - read, 1 - write
 *
 * where id is the event/output identifier.
 */
#define GCP_PSCHED_VALUE_SHIFT	0
#define GCP_PSCHED_VALUE_BITS	24
#define GCP_PSCHED_VALUE_MASK	((1 << GCP_PSCHED_VALUE_BITS) - 1)
#define GCP_PSCHED_ID_SHIFT	24
#define GCP_PSCHED_ID_BITS	7
#define GCP_PSCHED_ID_MASK	((1 << GCP_PSCHED_ID_BITS) - 1)
#define GCP_PSCHED_WRITE	0x80000000
 
#define GCP_CPSCHED_SYNC	0x010	/* position */
#define GCP_CPSCHED_START	0x011	/* position */
#define GCP_CPSCHED_MODE	0x012
#define GCP_CPSCHED_MODE_MANUAL	0
#define GCP_CPSCHED_MODE_TIMER	1
#define GCP_CPSCHED_MODE_POS	2
#define GCP_CPSCHED_MODE_MASK	0xf

#define GCP_CPSCHED_PERIOD	0x013	/* position in UQ8.16 */
#define GCP_CPSCHED_TIME_PERIOD	0x014	/* microseconds */
#define GCP_CPSCHED_SETUP_TIME	0x015	/* microseconds signed */
#define GCP_CPSCHED_SETUP_MAX	0x016	/* microseconds */
#define GCP_CPSCHED_HOLD_TIME	0x017	/* microseconds */
#define GCP_CPSCHED_SYNC_TIME	0x018	/* microseconds */
#define GCP_CPSCHED_BACKOFF_TIME 0x019	/* microseconds */
#define GCP_CPSCHED_START_AFTER	0x01a
#define GCP_CPSCHED_EVENT_DELAY	0x01b	/* microseconds */
#define GCP_CPSCHED_PERIOD_N	0x01c	/* position numerator */
#define GCP_CPSCHED_PERIOD_D	0x01d	/* position denominator */

#endif
